#############################################################################################################
# Variables
#############################################################################################################
variable "name" {}

variable "vpc_id" {
  description = "ID of the VPC in which the ELB should reside"
}

# Launch Configuration Variables
variable "ami" {
  description = "The AMI to use with the launch configuration"
}
variable "instance_type" {
  default = "m3.medium"
}
variable "iam_instance_profile" {
  description = "The IAM role the launched instance will use"
}

variable "security_groups" {
  description = "ID of SG the launched instance will use"
  type = "list"
}

variable "user_data" {
  description = "The path to a file with user_data for the instances"
}

# Auto-Scaling Group
variable "minimum_number_of_instances" {
  description = "The minimum number of instances we want in the ASG"
  default = "2"
}
variable "maximum_number_of_instances" {
  description = "The maximum number of instances we want in the ASG"
  default = "2"
}
variable "desired_number_of_instances" {
  description = "The desired number of instances we want in the ASG"
  default = "2"
}
variable "health_check_grace_period" {
  description = "Number of seconds for a health check to time out"
  default = 300
}
variable "health_check_type" {
  description = "The health check used by the ASG to determine health"
  default = "ELB"
}
variable "load_balancers" {
  description = "A list of ELB names the ASG should associate instances with"
  type = "list"
}
variable "vpc_zone_identifier" {
  type = "list"
}
variable "termination_policies" {
  type = "list"
  default = ["ClosestToNextInstanceHour", "OldestInstance"]
}

# Auto-Scaling policies
variable "scale_up_scaling_adjustment" {
  default = "50"
}
variable "scale_up_cooldown" {
  default = "60"
}
variable "scale_down_scaling_adjustment" {
  default = "-50"
}
variable "scale_down_cooldown" {
  default = "120"
}


# Cloudwatch vars

variable "cpu_high_threshold" {
  description = "CPU percentage that must be exceeded to trigger the alarm."
  default = "80"
}

variable "cpu_high_period" {
  description = "The period in seconds over which the specified statistic is applied."
  default = "60"
}

variable "cpu_high_evaluation_periods" {
  description = "The number of periods over which data is compared to the specified threshold."
  default = "2"
}

variable "cpu_low_threshold" {
  description = "CPU percentage that must be exceeded to trigger the alarm."
  default = "50"
}

variable "cpu_low_period" {
  description = "The period in seconds over which the specified statistic is applied."
  default = "300"
}

variable "cpu_low_evaluation_periods" {
  description = "The number of periods over which data is compared to the specified threshold."
  default = "3"
}


#############################################################################################################
# Security Group
#############################################################################################################
resource "aws_security_group" "main" {
  vpc_id      = "${var.vpc_id}"
  description = "Application EC2 SG"

  ingress {
    from_port   = -1
    to_port     = -1
    protocol    = "icmp"
    cidr_blocks = ["0.0.0.0/0"]
  }

  ingress {
    from_port       = 22
    to_port         = 22
    protocol        = "tcp"
    security_groups = ["${var.security_groups}"]
  }

  ingress {
    from_port       = 80 # Change to service port
    to_port         = 80
    protocol        = "tcp"
    security_groups = ["${var.security_groups}"]
  }

  egress {
    from_port   = 0
    to_port     = 0
    protocol    = "-1"
    cidr_blocks = ["0.0.0.0/0"]
  }
}

#############################################################################################################
# Launch Configuration
#############################################################################################################
resource "aws_launch_configuration" "main" {
  lifecycle { create_before_destroy = true }
  name_prefix          = "${var.name}-lc-"
  image_id             = "${var.ami}"
  instance_type        = "${var.instance_type}"
  iam_instance_profile = "${var.iam_instance_profile}"
  security_groups      = ["${var.security_groups}"]
  user_data            = "${var.user_data}"
  root_block_device {
    delete_on_termination = "true"
  }
}

#############################################################################################################
# Autoscaling Groups
#############################################################################################################
resource "aws_autoscaling_group" "main" {
  lifecycle { create_before_destroy = true }
  name                      = "${var.name} - ${aws_launch_configuration.main.name}"
  launch_configuration      = "${aws_launch_configuration.main.name}"
  load_balancers            = ["${var.load_balancers}"]
  vpc_zone_identifier       = ["${var.vpc_zone_identifier}"]
  min_elb_capacity          = "${var.minimum_number_of_instances}"
  min_size                  = "${var.minimum_number_of_instances}"
  max_size                  = "${var.maximum_number_of_instances}"
  desired_capacity          = "${var.desired_number_of_instances}"
  health_check_grace_period = "${var.health_check_grace_period}"
  health_check_type         = "${var.health_check_type}"
  termination_policies      = "${var.termination_policies}"
  tag {
    key                 = "Name"
    value               = "${var.name}"
    propagate_at_launch = true
  }
  tag {
    key                 = "Launch config"
    value               = "${aws_launch_configuration.main.name}"
    propagate_at_launch = true
  }
}

#############################################################################################################
# Autoscaling Policies
#############################################################################################################

resource "aws_autoscaling_policy" "scale_up" {
  name                   = "${var.name}-scale-up"
  adjustment_type        = "PercentChangeInCapacity"
  scaling_adjustment     = "${var.scale_up_scaling_adjustment}"
  cooldown               = "${var.scale_up_cooldown}"
  autoscaling_group_name = "${aws_autoscaling_group.main.name}"
}

resource "aws_autoscaling_policy" "scale_down" {
  name                   = "${var.name}-scale-down"
  adjustment_type        = "PercentChangeInCapacity"
  scaling_adjustment     = "${var.scale_down_scaling_adjustment}"
  cooldown               = "${var.scale_down_cooldown}"
  autoscaling_group_name = "${aws_autoscaling_group.main.name}"
}

#############################################################################################################
# Cloudwatch Alarms
#############################################################################################################

resource "aws_cloudwatch_metric_alarm" "cpu_high" {
  alarm_name          = "${var.name}-app-cpu-high"
  metric_name         = "CPUUtilization"
  namespace           = "AWS/EC2"
  statistic           = "Average"
  comparison_operator = "GreaterThanOrEqualToThreshold"
  threshold           = "${var.cpu_high_threshold}"
  period              = "${var.cpu_high_period}"
  evaluation_periods  = "${var.cpu_high_evaluation_periods}"
  alarm_description   = "This will alarm when an app servers CPU usage is above ${var.cpu_high_threshold}% for ${var.cpu_high_evaluation_periods} * ${var.cpu_high_period}"
  alarm_actions       = [
    "${aws_autoscaling_policy.scale_up.arn}"
  ]
  dimensions {
    AutoScalingGroupName = "${aws_autoscaling_group.main.id}"
  }
}

resource "aws_cloudwatch_metric_alarm" "cpu_low" {
  alarm_name          = "${var.name}-app-cpu-low"
  metric_name         = "CPUUtilization"
  namespace           = "AWS/EC2"
  statistic           = "Average"
  comparison_operator = "LessThanOrEqualToThreshold"
  threshold           = "${var.cpu_low_threshold}"
  period              = "${var.cpu_low_period}"
  evaluation_periods  = "${var.cpu_low_evaluation_periods}"
  alarm_description   = "This will alarm when an app servers CPU usage is below ${var.cpu_low_threshold}% for ${var.cpu_low_evaluation_periods} * ${var.cpu_low_period}"
  alarm_actions       = [
    "${aws_autoscaling_policy.scale_down.arn}"
  ]
  dimensions {
    AutoScalingGroupName = "${aws_autoscaling_group.main.id}"
  }
}


#############################################################################################################
# Outputs
#############################################################################################################
output "launch_config_id" {
    value = "${aws_launch_configuration.main.id}"
}
output "asg_id" {
    value = "${aws_autoscaling_group.main.id}"
}
output "asg_name" {
    value = "${aws_autoscaling_group.main.name}"
}
output "autoscaling_policy_scale_up_arn" {
  value = "${aws_autoscaling_policy.scale_up.arn}"
}
output "autoscaling_policy_scale_down_arn" {
  value = "${aws_autoscaling_policy.scale_down.arn}"
}
output "security_group" {
  value = "${aws_security_group.main.id}"
}
